/**
 * ==============================================
 *  LibReDE : Library for Resource Demand Estimation
 * ==============================================
 *
 * (c) Copyright 2013-2014, by Simon Spinner and Contributors.
 *
 * Project Info:   http://www.descartes-research.net/
 *
 * All rights reserved. This software is made available under the terms of the
 * Eclipse Public License (EPL) v1.0 as published by the Eclipse Foundation
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License (EPL)
 * for more details.
 *
 * You should have received a copy of the Eclipse Public License (EPL)
 * along with this software; if not visit http://www.eclipse.org or write to
 * Eclipse Foundation, Inc., 308 SW First Avenue, Suite 110, Portland, 97204 USA
 * Email: license (at) eclipse.org
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */
package tools.descartes.prisma.agent.wildfly.interceptors;

import java.util.Deque;

import org.jboss.logging.Logger;

import io.undertow.server.HttpHandler;
import io.undertow.server.HttpServerExchange;
import tools.descartes.prisma.agent.wildfly.callstack.ComponentRecord;
import tools.descartes.prisma.agent.wildfly.callstack.InvocationRecord;
import tools.descartes.prisma.agent.wildfly.callstack.OperationRecord;
import tools.descartes.prisma.agent.wildfly.service.MonitoringService;

public class IncomingServletCallHandler implements HttpHandler {
	
	private static final Logger log = Logger.getLogger(IncomingServletCallHandler.class);
	
	private HttpHandler next;
	
	private String serverNode;

	private MonitoringService monitoringService;

	public IncomingServletCallHandler(HttpHandler next) {
		this.next = next;
		this.serverNode = System.getProperty("jboss.node.name");
		this.monitoringService = MonitoringService.INSTANCE;
	}
	
	@Override
	public void handleRequest(HttpServerExchange req) throws Exception {
		if (req.isInIoThread()) {
			req.dispatch(this);
			return;
	    }
		
		Deque<InvocationRecord> threadCallStack = monitoringService.getCallStack();

		// Check whether this is a soap request. The SOAPAction field needs to
		// be in the HTTP header and can be used to identify SOAP requests. The
		// information on service/operation/... will be filled in a later step
		// when the WSDL is parsed.
		if (!req.getRequestHeaders().contains("SOAPAction")) {
			ComponentRecord component = monitoringService.getComponent(serverNode + req.getRequestPath());
			if (component != null) {
				String interf = component.getComponentClass();
				Deque<String> actionParamter = req.getQueryParameters().get("action");
				if (actionParamter != null) {
					String operation = actionParamter.getFirst();
					OperationRecord orec = monitoringService.requireOperation(component, interf,
							"http", operation);
					InvocationRecord invocation = startInvocation(orec, req);
					threadCallStack.push(invocation);
				}
			}
		}
		next.handleRequest(req);

		long endTime = System.nanoTime();
		if (!threadCallStack.isEmpty()) {
			long startTime = req.getRequestStartTime();
			InvocationRecord invocation = threadCallStack.pop();
			long executionTime = endTime - startTime;
			if (log.isDebugEnabled()) {
				log.debug("Incoming invocation ("
						+ invocation.getOperation().getComponent().getComponentName() + ", "
						+ invocation.getOperation().getOperationName()
						+ ", " + executionTime + "ms)");
			}
			invocation.finishInvocation(executionTime);
		}
	}

	
	private InvocationRecord startInvocation(OperationRecord orec, HttpServerExchange req) {
		String[] names = orec.getObservedParameterNames();
		if (names.length > 0) {
			Deque<String> params = req.getQueryParameters().get(names[0]);
			if (params != null && !params.isEmpty()) {
				String value = params.getFirst();
				return orec.startInvocation((value != null && value.isEmpty()) ? (Object) null : value);
			} else {
				return orec.startInvocation((Object) null);
			}			
		}
		return orec.startInvocation();
	}

}
